/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/autoAddFriendIdsAsContact",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsIdentity->autoAddFriendIdsAsContact();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/createIdentity",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;
		std::string name;
		RsGxsImage avatar = RsGxsImage();
		bool pseudonimous = true;
		std::string pgpPassword = "";


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(name);
			RS_SERIAL_PROCESS(avatar);
			RS_SERIAL_PROCESS(pseudonimous);
			RS_SERIAL_PROCESS(pgpPassword);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->createIdentity(id, name, avatar, pseudonimous, pgpPassword);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/deleteBannedNodesThreshold",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsIdentity->deleteBannedNodesThreshold();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/deleteIdentity",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->deleteIdentity(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/exportIdentityLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		std::string link;
		RsGxsId id;
		bool includeGxsData = true;
		std::string baseUrl = RsIdentity::DEFAULT_IDENTITY_BASE_URL;
		std::string errMsg = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(includeGxsData);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->exportIdentityLink(link, id, includeGxsData, baseUrl, errMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(errMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getDefaultStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsIdentity->getDefaultStoragePeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getDefaultSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsIdentity->getDefaultSyncPeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getIdDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;
		RsIdentityDetails details;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->getIdDetails(id, details);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(details);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getIdentitiesInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		std::set<RsGxsId> ids;
		std::vector<RsGxsIdGroup> idsInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(ids);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->getIdentitiesInfo(ids, idsInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(idsInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getIdentitiesSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		std::list<RsGroupMetaData> ids;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsIdentity->getIdentitiesSummaries(ids);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(ids);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getLastUsageTS",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		rstime_t retval = rsIdentity->getLastUsageTS(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getOwnPseudonimousIds",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		std::vector<RsGxsId> ids;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsIdentity->getOwnPseudonimousIds(ids);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(ids);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getOwnSignedIds",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		std::vector<RsGxsId> ids;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsIdentity->getOwnSignedIds(ids);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(ids);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsIdentity->getStoragePeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/getSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsIdentity->getSyncPeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/isARegularContact",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->isARegularContact(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/isKnownId",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->isKnownId(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/isOwnId",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->isOwnId(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/requestIdentity",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;
		std::vector<RsPeerId> peers = std::vector< RsPeerId >();


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(peers);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->requestIdentity(id, peers);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/requestStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		uint32_t token;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(token);
		}


		// call retroshare C++ API
		RsTokenService::GxsRequestStatus retval = rsIdentity->requestStatus(token);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/setAsRegularContact",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;
		bool isContact;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(isContact);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->setAsRegularContact(id, isContact);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/setAutoAddFriendIdsAsContact",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		bool enabled;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(enabled);
		}


		// call retroshare C++ API
		rsIdentity->setAutoAddFriendIdsAsContact(enabled);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/setDeleteBannedNodesThreshold",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		uint32_t days;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(days);
		}


		// call retroshare C++ API
		rsIdentity->setDeleteBannedNodesThreshold(days);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/setStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t storageSecs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(storageSecs);
		}


		// call retroshare C++ API
		rsIdentity->setStoragePeriod(groupId, storageSecs);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/setSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t syncAge;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(syncAge);
		}


		// call retroshare C++ API
		rsIdentity->setSyncPeriod(groupId, syncAge);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsIdentity/updateIdentity",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsIdentity, "rsIdentity", cAns, session ) )
			return;

		RsGxsId id;
		std::string name;
		RsGxsImage avatar;
		bool pseudonimous;
		std::string pgpPassword;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(name);
			RS_SERIAL_PROCESS(avatar);
			RS_SERIAL_PROCESS(pseudonimous);
			RS_SERIAL_PROCESS(pgpPassword);
		}


		// call retroshare C++ API
		bool retval = rsIdentity->updateIdentity(id, name, avatar, pseudonimous, pgpPassword);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsControl/isReady",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsControl, "rsControl", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsControl->isReady();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsLoginHelper/attemptLogin",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsLoginHelper, "rsLoginHelper", cAns, session ) )
			return;

		RsPeerId account;
		std::string password;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(account);
			RS_SERIAL_PROCESS(password);
		}


		// call retroshare C++ API
		RsInit::LoadCertificateStatus retval = rsLoginHelper->attemptLogin(account, password);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsLoginHelper/collectEntropy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsLoginHelper, "rsLoginHelper", cAns, session ) )
			return;

		uint32_t bytes;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(bytes);
		}


		// call retroshare C++ API
		bool retval = rsLoginHelper->collectEntropy(bytes);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsLoginHelper/getLocations",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsLoginHelper, "rsLoginHelper", cAns, session ) )
			return;

		std::vector<RsLoginHelper::Location> locations;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsLoginHelper->getLocations(locations);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(locations);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsLoginHelper/isLoggedIn",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsLoginHelper, "rsLoginHelper", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsLoginHelper->isLoggedIn();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/ConfigDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::string retval = rsAccounts->ConfigDirectory();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/ExportIdentity",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;

		std::string filePath;
		RsPgpId pgpId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(filePath);
			RS_SERIAL_PROCESS(pgpId);
		}


		// call retroshare C++ API
		bool retval = rsAccounts->ExportIdentity(filePath, pgpId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/exportIdentityToString",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;

		std::string data;
		RsPgpId pgpId;
		std::string errorMsg;
		bool includeSignatures = true;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(pgpId);
			RS_SERIAL_PROCESS(includeSignatures);
		}


		// call retroshare C++ API
		bool retval = rsAccounts->exportIdentityToString(data, pgpId, errorMsg, includeSignatures);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(data);
			RS_SERIAL_PROCESS(errorMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/getCurrentAccountId",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;

		RsPeerId id;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsAccounts->getCurrentAccountId(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/GetPGPLogins",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;

		std::list<RsPgpId> pgpIds;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsAccounts->GetPGPLogins(pgpIds);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(pgpIds);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/ImportIdentity",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;

		std::string filePath;
		RsPgpId pgpId;
		std::string errorMsg;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(filePath);
		}


		// call retroshare C++ API
		bool retval = rsAccounts->ImportIdentity(filePath, pgpId, errorMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(pgpId);
			RS_SERIAL_PROCESS(errorMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsAccounts/importIdentityFromString",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsAccounts, "rsAccounts", cAns, session ) )
			return;

		std::string data;
		RsPgpId pgpId;
		std::string errorMsg;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(data);
		}


		// call retroshare C++ API
		bool retval = rsAccounts->importIdentityFromString(data, pgpId, errorMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(pgpId);
			RS_SERIAL_PROCESS(errorMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGossipDiscovery/getDiscFriends",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGossipDiscovery, "rsGossipDiscovery", cAns, session ) )
			return;

		RsPeerId id;
		std::list<RsPeerId> friends;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsGossipDiscovery->getDiscFriends(id, friends);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(friends);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGossipDiscovery/getDiscPgpFriends",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGossipDiscovery, "rsGossipDiscovery", cAns, session ) )
			return;

		RsPgpId pgpid;
		std::list<RsPgpId> gpg_friends;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(pgpid);
		}


		// call retroshare C++ API
		bool retval = rsGossipDiscovery->getDiscPgpFriends(pgpid, gpg_friends);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(gpg_friends);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGossipDiscovery/getPeerVersion",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGossipDiscovery, "rsGossipDiscovery", cAns, session ) )
			return;

		RsPeerId id;
		std::string version;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsGossipDiscovery->getPeerVersion(id, version);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(version);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGossipDiscovery/getWaitingDiscCount",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGossipDiscovery, "rsGossipDiscovery", cAns, session ) )
			return;

		size_t sendCount;
		size_t recvCount;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsGossipDiscovery->getWaitingDiscCount(sendCount, recvCount);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(sendCount);
			RS_SERIAL_PROCESS(recvCount);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/createForum",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsForumGroup forum;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forum);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->createForum(forum);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(forum);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/createForumV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		std::string name;
		std::string description;
		RsGxsId authorId = RsGxsId();
		std::set<RsGxsId> moderatorsIds = std::set< RsGxsId >();
		RsGxsCircleType circleType = RsGxsCircleType::PUBLIC;
		RsGxsCircleId circleId = RsGxsCircleId();
		RsGxsGroupId forumId = RS_DEFAULT_STORAGE_PARAM(RsGxsGroupId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(name);
			RS_SERIAL_PROCESS(description);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(moderatorsIds);
			RS_SERIAL_PROCESS(circleType);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->createForumV2(name, description, authorId, moderatorsIds, circleType, circleId, forumId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/createMessage",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsForumMsg message;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(message);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->createMessage(message);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(message);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/createPost",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		std::string title;
		std::string mBody;
		RsGxsId authorId;
		RsGxsMessageId parentId = RsGxsMessageId();
		RsGxsMessageId origPostId = RsGxsMessageId();
		RsGxsMessageId postMsgId = RS_DEFAULT_STORAGE_PARAM(RsGxsMessageId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(title);
			RS_SERIAL_PROCESS(mBody);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(parentId);
			RS_SERIAL_PROCESS(origPostId);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->createPost(forumId, title, mBody, authorId, parentId, origPostId, postMsgId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(postMsgId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/distantSearchRequest",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		std::string matchString;
		TurtleRequestId searchId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(matchString);
		}


		// call retroshare C++ API
		std::error_condition retval = rsGxsForums->distantSearchRequest(matchString, searchId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(searchId);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/editForum",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsForumGroup forum;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forum);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->editForum(forum);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/exportForumLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		std::string link;
		RsGxsGroupId forumId;
		bool includeGxsData = true;
		std::string baseUrl = RsGxsForums::DEFAULT_FORUM_BASE_URL;
		std::string errMsg = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(includeGxsData);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->exportForumLink(link, forumId, includeGxsData, baseUrl, errMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(errMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getChildPosts",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		RsGxsMessageId parentId;
		std::vector<RsGxsForumMsg> childPosts;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(parentId);
		}


		// call retroshare C++ API
		std::error_condition retval = rsGxsForums->getChildPosts(forumId, parentId, childPosts);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(childPosts);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getContentSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		std::set<RsGxsMessageId> contentIds;
		std::vector<RsMsgMetaData> summaries;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(contentIds);
		}


		// call retroshare C++ API
		std::error_condition retval = rsGxsForums->getContentSummaries(forumId, contentIds, summaries);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(summaries);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getDefaultStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsGxsForums->getDefaultStoragePeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getDefaultSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsGxsForums->getDefaultSyncPeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumContent",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		std::set<RsGxsMessageId> msgsIds;
		std::vector<RsGxsForumMsg> msgs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(msgsIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumContent(forumId, msgsIds, msgs);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(msgs);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumGroupStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		GxsGroupStatistic stat;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumGroupStatistics(forumId, stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumMsgMetaData",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		std::vector<RsMsgMetaData> msgMetas;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumMsgMetaData(forumId, msgMetas);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(msgMetas);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumPostsHierarchy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsForumGroup group;
		std::vector<ForumPostEntry> vect;
		std::map<RsGxsMessageId,std::vector<std::pair<rstime_t,RsGxsMessageId>>> post_versions;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(group);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumPostsHierarchy(group, vect, post_versions);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(vect);
			RS_SERIAL_PROCESS(post_versions);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumServiceStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		GxsServiceStatistic stat;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumServiceStatistics(stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumsInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		std::list<RsGxsGroupId> forumIds;
		std::vector<RsGxsForumGroup> forumsInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumsInfo(forumIds, forumsInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(forumsInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumsSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		std::list<RsGroupMetaData> forums;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumsSummaries(forums);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(forums);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getForumStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		RsGxsForumStatistics stat;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->getForumStatistics(forumId, stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsGxsForums->getStoragePeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/getSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsGxsForums->getSyncPeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/localSearch",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		std::string matchString;
		std::vector<RsGxsSearchResult> searchResults;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(matchString);
		}


		// call retroshare C++ API
		std::error_condition retval = rsGxsForums->localSearch(matchString, searchResults);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(searchResults);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/markRead",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGrpMsgIdPair messageId;
		bool read;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(messageId);
			RS_SERIAL_PROCESS(read);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->markRead(messageId, read);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/requestStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		uint32_t token;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(token);
		}


		// call retroshare C++ API
		RsTokenService::GxsRequestStatus retval = rsGxsForums->requestStatus(token);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/requestSynchronization",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::error_condition retval = rsGxsForums->requestSynchronization();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/setPostKeepForever",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		RsGxsMessageId postId;
		bool keepForever;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(postId);
			RS_SERIAL_PROCESS(keepForever);
		}


		// call retroshare C++ API
		std::error_condition retval = rsGxsForums->setPostKeepForever(forumId, postId, keepForever);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/setStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t storageSecs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(storageSecs);
		}


		// call retroshare C++ API
		rsGxsForums->setStoragePeriod(groupId, storageSecs);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/setSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t syncAge;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(syncAge);
		}


		// call retroshare C++ API
		rsGxsForums->setSyncPeriod(groupId, syncAge);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/subscribeToForum",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		RsGxsGroupId forumId;
		bool subscribe;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forumId);
			RS_SERIAL_PROCESS(subscribe);
		}


		// call retroshare C++ API
		bool retval = rsGxsForums->subscribeToForum(forumId, subscribe);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsForums/updateReputationLevel",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsForums, "rsGxsForums", cAns, session ) )
			return;

		uint32_t forum_group_sign_flags;
		ForumPostEntry e;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(forum_group_sign_flags);
			RS_SERIAL_PROCESS(e);
		}


		// call retroshare C++ API
		rsGxsForums->updateReputationLevel(forum_group_sign_flags, e);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(e);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsBanList/enableIPFiltering",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsBanList, "rsBanList", cAns, session ) )
			return;

		bool enable;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(enable);
		}


		// call retroshare C++ API
		rsBanList->enableIPFiltering(enable);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsBanList/ipFilteringEnabled",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsBanList, "rsBanList", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsBanList->ipFilteringEnabled();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsBroadcastDiscovery/disableMulticastListening",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsBroadcastDiscovery, "rsBroadcastDiscovery", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsBroadcastDiscovery->disableMulticastListening();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsBroadcastDiscovery/enableMulticastListening",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsBroadcastDiscovery, "rsBroadcastDiscovery", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsBroadcastDiscovery->enableMulticastListening();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsBroadcastDiscovery/getDiscoveredPeers",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsBroadcastDiscovery, "rsBroadcastDiscovery", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::vector< RsBroadcastDiscoveryResult > retval = rsBroadcastDiscovery->getDiscoveredPeers();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsBroadcastDiscovery/isMulticastListeningEnabled",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsBroadcastDiscovery, "rsBroadcastDiscovery", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsBroadcastDiscovery->isMulticastListeningEnabled();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getOwnServices",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		RsPeerServiceInfo info;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsServiceControl->getOwnServices(info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getPeersConnected",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		uint32_t serviceId;
		std::set<RsPeerId> peerSet;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(serviceId);
		}


		// call retroshare C++ API
		rsServiceControl->getPeersConnected(serviceId, peerSet);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(peerSet);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getServiceItemNames",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		uint32_t serviceId;
		std::map<uint8_t,std::string> names;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(serviceId);
		}


		// call retroshare C++ API
		bool retval = rsServiceControl->getServiceItemNames(serviceId, names);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(names);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getServiceName",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		uint32_t serviceId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(serviceId);
		}


		// call retroshare C++ API
		std::string retval = rsServiceControl->getServiceName(serviceId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getServicePermissions",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		uint32_t serviceId;
		RsServicePermissions permissions;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(serviceId);
		}


		// call retroshare C++ API
		bool retval = rsServiceControl->getServicePermissions(serviceId, permissions);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(permissions);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getServicesAllowed",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		RsPeerId peerId;
		RsPeerServiceInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(peerId);
		}


		// call retroshare C++ API
		bool retval = rsServiceControl->getServicesAllowed(peerId, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/getServicesProvided",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		RsPeerId peerId;
		RsPeerServiceInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(peerId);
		}


		// call retroshare C++ API
		bool retval = rsServiceControl->getServicesProvided(peerId, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsServiceControl/updateServicePermissions",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsServiceControl, "rsServiceControl", cAns, session ) )
			return;

		uint32_t serviceId;
		RsServicePermissions permissions;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(serviceId);
			RS_SERIAL_PROCESS(permissions);
		}


		// call retroshare C++ API
		bool retval = rsServiceControl->updateServicePermissions(serviceId, permissions);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/createBoard",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsPostedGroup board;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(board);
		}


		// call retroshare C++ API
		bool retval = rsPosted->createBoard(board);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(board);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/createBoardV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		std::string board_name;
		std::string board_description;
		RsGxsImage board_image;
		RsGxsId authorId;
		RsGxsCircleType circleType;
		RsGxsCircleId circleId;
		RsGxsGroupId boardId;
		std::string errorMessage;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(board_name);
			RS_SERIAL_PROCESS(board_description);
			RS_SERIAL_PROCESS(board_image);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(circleType);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsPosted->createBoardV2(board_name, board_description, board_image, authorId, circleType, circleId, boardId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(boardId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/createCommentV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		RsGxsMessageId postId;
		std::string comment;
		RsGxsId authorId;
		RsGxsMessageId parentId = RsGxsMessageId();
		RsGxsMessageId origCommentId = RsGxsMessageId();
		RsGxsMessageId commentMessageId = RS_DEFAULT_STORAGE_PARAM(RsGxsMessageId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
			RS_SERIAL_PROCESS(postId);
			RS_SERIAL_PROCESS(comment);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(parentId);
			RS_SERIAL_PROCESS(origCommentId);
		}


		// call retroshare C++ API
		bool retval = rsPosted->createCommentV2(boardId, postId, comment, authorId, parentId, origCommentId, commentMessageId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(commentMessageId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/createPost",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsPostedPost post;
		RsGxsMessageId post_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(post);
		}


		// call retroshare C++ API
		bool retval = rsPosted->createPost(post, post_id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(post_id);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/createPostV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		std::string title;
		RsUrl link;
		std::string notes;
		RsGxsId authorId;
		RsGxsImage image;
		RsGxsMessageId postId;
		std::string error_message;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
			RS_SERIAL_PROCESS(title);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(notes);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(image);
		}


		// call retroshare C++ API
		bool retval = rsPosted->createPostV2(boardId, title, link, notes, authorId, image, postId, error_message);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(postId);
			RS_SERIAL_PROCESS(error_message);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/editBoard",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsPostedGroup board;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(board);
		}


		// call retroshare C++ API
		bool retval = rsPosted->editBoard(board);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getBoardAllContent",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		std::vector<RsPostedPost> posts;
		std::vector<RsGxsComment> comments;
		std::vector<RsGxsVote> votes;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
		}


		// call retroshare C++ API
		bool retval = rsPosted->getBoardAllContent(boardId, posts, comments, votes);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(posts);
			RS_SERIAL_PROCESS(comments);
			RS_SERIAL_PROCESS(votes);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getBoardContent",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		std::set<RsGxsMessageId> contentsIds;
		std::vector<RsPostedPost> posts;
		std::vector<RsGxsComment> comments;
		std::vector<RsGxsVote> votes;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
			RS_SERIAL_PROCESS(contentsIds);
		}


		// call retroshare C++ API
		bool retval = rsPosted->getBoardContent(boardId, contentsIds, posts, comments, votes);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(posts);
			RS_SERIAL_PROCESS(comments);
			RS_SERIAL_PROCESS(votes);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getBoardsInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		std::list<RsGxsGroupId> boardsIds;
		std::vector<RsPostedGroup> boardsInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardsIds);
		}


		// call retroshare C++ API
		bool retval = rsPosted->getBoardsInfo(boardsIds, boardsInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(boardsInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getBoardsServiceStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		GxsServiceStatistic stat;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsPosted->getBoardsServiceStatistics(stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getBoardsSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		std::list<RsGroupMetaData> groupInfo;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsPosted->getBoardsSummaries(groupInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(groupInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getBoardStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		GxsGroupStatistic stat;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
		}


		// call retroshare C++ API
		bool retval = rsPosted->getBoardStatistics(boardId, stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getDefaultStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsPosted->getDefaultStoragePeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getDefaultSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsPosted->getDefaultSyncPeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsPosted->getStoragePeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/getSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsPosted->getSyncPeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/requestStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		uint32_t token;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(token);
		}


		// call retroshare C++ API
		RsTokenService::GxsRequestStatus retval = rsPosted->requestStatus(token);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/setCommentReadStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGrpMsgIdPair msgId;
		bool read;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(read);
		}


		// call retroshare C++ API
		bool retval = rsPosted->setCommentReadStatus(msgId, read);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/setPostReadStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGrpMsgIdPair msgId;
		bool read;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(read);
		}


		// call retroshare C++ API
		bool retval = rsPosted->setPostReadStatus(msgId, read);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/setStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t storageSecs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(storageSecs);
		}


		// call retroshare C++ API
		rsPosted->setStoragePeriod(groupId, storageSecs);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/setSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t syncAge;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(syncAge);
		}


		// call retroshare C++ API
		rsPosted->setSyncPeriod(groupId, syncAge);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/subscribeToBoard",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		bool subscribe;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
			RS_SERIAL_PROCESS(subscribe);
		}


		// call retroshare C++ API
		bool retval = rsPosted->subscribeToBoard(boardId, subscribe);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/voteForComment",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId boardId;
		RsGxsMessageId postId;
		RsGxsMessageId commentId;
		RsGxsId authorId;
		RsGxsVoteType vote;
		RsGxsMessageId voteId;
		std::string errorMessage;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(boardId);
			RS_SERIAL_PROCESS(postId);
			RS_SERIAL_PROCESS(commentId);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(vote);
		}


		// call retroshare C++ API
		bool retval = rsPosted->voteForComment(boardId, postId, commentId, authorId, vote, voteId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(voteId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPosted/voteForPost",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPosted, "rsPosted", cAns, session ) )
			return;

		RsGxsGroupId postGrpId;
		RsGxsMessageId postMsgId;
		RsGxsId authorId;
		RsGxsVoteType vote;
		RsGxsMessageId voteId;
		std::string errorMessage;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(postGrpId);
			RS_SERIAL_PROCESS(postMsgId);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(vote);
		}


		// call retroshare C++ API
		bool retval = rsPosted->voteForPost(postGrpId, postMsgId, authorId, vote, voteId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(voteId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/askForStop",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsJsonApi->askForStop();


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/authorizeUser",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		std::string user;
		std::string password;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(user);
			RS_SERIAL_PROCESS(password);
		}


		// call retroshare C++ API
		std::error_condition retval = rsJsonApi->authorizeUser(user, password);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/getAuthorizedTokens",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::map< std::string, std::string > retval = rsJsonApi->getAuthorizedTokens();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/getBindingAddress",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::string retval = rsJsonApi->getBindingAddress();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/isAuthTokenValid",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		std::string token;
		std::error_condition error = RS_DEFAULT_STORAGE_PARAM(std::error_condition);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(token);
		}


		// call retroshare C++ API
		bool retval = rsJsonApi->isAuthTokenValid(token, error);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(error);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/listeningPort",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint16_t retval = rsJsonApi->listeningPort();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/requestNewTokenAutorization",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		std::string user;
		std::string password;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(user);
			RS_SERIAL_PROCESS(password);
		}


		// call retroshare C++ API
		std::error_condition retval = rsJsonApi->requestNewTokenAutorization(user, password);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/revokeAuthToken",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		std::string user;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(user);
		}


		// call retroshare C++ API
		bool retval = rsJsonApi->revokeAuthToken(user);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/setBindingAddress",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		std::string address;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(address);
		}


		// call retroshare C++ API
		rsJsonApi->setBindingAddress(address);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/setListeningPort",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		uint16_t port;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(port);
		}


		// call retroshare C++ API
		rsJsonApi->setListeningPort(port);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsJsonApi/version",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsJsonApi, "rsJsonApi", cAns, session ) )
			return;

		uint32_t major;
		uint32_t minor;
		uint32_t mini;
		std::string extra;
		std::string human;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsJsonApi->version(major, minor, mini, extra, human);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(major);
			RS_SERIAL_PROCESS(minor);
			RS_SERIAL_PROCESS(mini);
			RS_SERIAL_PROCESS(extra);
			RS_SERIAL_PROCESS(human);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, false );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/cancelCircleMembership",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsId ownGxsId;
		RsGxsCircleId circleId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(ownGxsId);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->cancelCircleMembership(ownGxsId, circleId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/createCircle",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::string circleName;
		RsGxsCircleType circleType;
		RsGxsCircleId circleId = RS_DEFAULT_STORAGE_PARAM(RsGxsCircleId);
		RsGxsCircleId restrictedId = RsGxsCircleId();
		RsGxsId authorId = RsGxsId();
		std::set<RsGxsId> gxsIdMembers = std::set< RsGxsId >();
		std::set<RsPgpId> localMembers = std::set< RsPgpId >();


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(circleName);
			RS_SERIAL_PROCESS(circleType);
			RS_SERIAL_PROCESS(restrictedId);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(gxsIdMembers);
			RS_SERIAL_PROCESS(localMembers);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->createCircle(circleName, circleType, circleId, restrictedId, authorId, gxsIdMembers, localMembers);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(circleId);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/editCircle",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsCircleGroup cData;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(cData);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->editCircle(cData);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(cData);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/exportCircleLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::string link;
		RsGxsCircleId circleId;
		bool includeGxsData = true;
		std::string baseUrl = RsGxsCircles::DEFAULT_CIRCLE_BASE_URL;
		std::string errMsg = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(circleId);
			RS_SERIAL_PROCESS(includeGxsData);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->exportCircleLink(link, circleId, includeGxsData, baseUrl, errMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(errMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getCircleDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsCircleId id;
		RsGxsCircleDetails details;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->getCircleDetails(id, details);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(details);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getCircleExternalIdList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::set<RsGxsCircleId> circleIds;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(circleIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->getCircleExternalIdList(circleIds);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getCircleRequest",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsGroupId circleId;
		RsGxsMessageId msgId;
		RsGxsCircleMsg msg;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(circleId);
			RS_SERIAL_PROCESS(msgId);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->getCircleRequest(circleId, msgId, msg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(msg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getCircleRequests",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsGroupId circleId;
		std::vector<RsGxsCircleMsg> requests;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->getCircleRequests(circleId, requests);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(requests);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getCirclesInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::list<RsGxsGroupId> circlesIds;
		std::vector<RsGxsCircleGroup> circlesInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(circlesIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->getCirclesInfo(circlesIds, circlesInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(circlesInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getCirclesSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::list<RsGroupMetaData> circles;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsGxsCircles->getCirclesSummaries(circles);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(circles);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getDefaultStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsGxsCircles->getDefaultStoragePeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getDefaultSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsGxsCircles->getDefaultSyncPeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsGxsCircles->getStoragePeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/getSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsGxsCircles->getSyncPeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/inviteIdsToCircle",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::set<RsGxsId> identities;
		RsGxsCircleId circleId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(identities);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->inviteIdsToCircle(identities, circleId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/requestCircleMembership",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsId ownGxsId;
		RsGxsCircleId circleId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(ownGxsId);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->requestCircleMembership(ownGxsId, circleId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/requestStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		uint32_t token;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(token);
		}


		// call retroshare C++ API
		RsTokenService::GxsRequestStatus retval = rsGxsCircles->requestStatus(token);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/revokeIdsFromCircle",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		std::set<RsGxsId> identities;
		RsGxsCircleId circleId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(identities);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsCircles->revokeIdsFromCircle(identities, circleId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/setStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t storageSecs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(storageSecs);
		}


		// call retroshare C++ API
		rsGxsCircles->setStoragePeriod(groupId, storageSecs);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsCircles/setSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsCircles, "rsGxsCircles", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t syncAge;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(syncAge);
		}


		// call retroshare C++ API
		rsGxsCircles->setSyncPeriod(groupId, syncAge);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/acceptLobbyInvite",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId id;
		RsGxsId identity;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(identity);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->acceptLobbyInvite(id, identity);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/clearChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		rsMsgs->clearChatLobby(id);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/closeDistantChatConnexion",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		DistantChatPeerId pid;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(pid);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->closeDistantChatConnexion(pid);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/createChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string lobby_name;
		RsGxsId lobby_identity;
		std::string lobby_topic;
		std::set<RsPeerId> invited_friends;
		ChatLobbyFlags lobby_privacy_type;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_name);
			RS_SERIAL_PROCESS(lobby_identity);
			RS_SERIAL_PROCESS(lobby_topic);
			RS_SERIAL_PROCESS(invited_friends);
			RS_SERIAL_PROCESS(lobby_privacy_type);
		}


		// call retroshare C++ API
		ChatLobbyId retval = rsMsgs->createChatLobby(lobby_name, lobby_identity, lobby_topic, invited_friends, lobby_privacy_type);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/denyLobbyInvite",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->denyLobbyInvite(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getChatLobbyInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId id;
		ChatLobbyInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getChatLobbyInfo(id, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getChatLobbyList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::list<ChatLobbyId> cl_list;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsMsgs->getChatLobbyList(cl_list);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(cl_list);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getCustomStateString",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		RsPeerId peer_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(peer_id);
		}


		// call retroshare C++ API
		std::string retval = rsMsgs->getCustomStateString(peer_id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getDefaultIdentityForChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsMsgs->getDefaultIdentityForChatLobby(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getDistantChatStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		DistantChatPeerId pid;
		DistantChatPeerInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(pid);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getDistantChatStatus(pid, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getDistantMessagingPermissionFlags",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsMsgs->getDistantMessagingPermissionFlags();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getIdentityForChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;
		RsGxsId nick;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getIdentityForChatLobby(lobby_id, nick);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(nick);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getListOfNearbyChatLobbies",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::vector<VisibleChatLobbyRecord> public_lobbies;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsMsgs->getListOfNearbyChatLobbies(public_lobbies);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(public_lobbies);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getLobbyAutoSubscribe",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getLobbyAutoSubscribe(lobby_id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMaxMessageSecuritySize",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		int type;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(type);
		}


		// call retroshare C++ API
		uint32_t retval = rsMsgs->getMaxMessageSecuritySize(type);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMessage",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		Rs::Msgs::MessageInfo msg;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getMessage(msgId, msg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(msg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMessageCount",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		uint32_t nInbox;
		uint32_t nInboxNew;
		uint32_t nOutbox;
		uint32_t nDraftbox;
		uint32_t nSentbox;
		uint32_t nTrashbox;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsMsgs->getMessageCount(nInbox, nInboxNew, nOutbox, nDraftbox, nSentbox, nTrashbox);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(nInbox);
			RS_SERIAL_PROCESS(nInboxNew);
			RS_SERIAL_PROCESS(nOutbox);
			RS_SERIAL_PROCESS(nDraftbox);
			RS_SERIAL_PROCESS(nSentbox);
			RS_SERIAL_PROCESS(nTrashbox);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMessageSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		Rs::Msgs::BoxName box;
		std::list<Rs::Msgs::MsgInfoSummary> msgList;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(box);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getMessageSummaries(box, msgList);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(msgList);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMessageTag",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		Rs::Msgs::MsgTagInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getMessageTag(msgId, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMessageTagTypes",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		Rs::Msgs::MsgTagType tags;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsMsgs->getMessageTagTypes(tags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(tags);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getMsgParentId",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		std::string msgParentId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->getMsgParentId(msgId, msgParentId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(msgParentId);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/getPendingChatLobbyInvites",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::list<ChatLobbyInvite> invites;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsMsgs->getPendingChatLobbyInvites(invites);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(invites);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/initiateDistantChatConnexion",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		RsGxsId to_pid;
		RsGxsId from_pid;
		DistantChatPeerId pid;
		uint32_t error_code;
		bool notify = true;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(to_pid);
			RS_SERIAL_PROCESS(from_pid);
			RS_SERIAL_PROCESS(notify);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->initiateDistantChatConnexion(to_pid, from_pid, pid, error_code, notify);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(pid);
			RS_SERIAL_PROCESS(error_code);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/invitePeerToLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;
		RsPeerId peer_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
			RS_SERIAL_PROCESS(peer_id);
		}


		// call retroshare C++ API
		rsMsgs->invitePeerToLobby(lobby_id, peer_id);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/joinVisibleChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;
		RsGxsId own_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
			RS_SERIAL_PROCESS(own_id);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->joinVisibleChatLobby(lobby_id, own_id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageDelete",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageDelete(msgId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageForwarded",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool forwarded;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(forwarded);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageForwarded(msgId, forwarded);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageJunk",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool mark;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(mark);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageJunk(msgId, mark);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageLoadEmbeddedImages",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool load;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(load);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageLoadEmbeddedImages(msgId, load);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageRead",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool unreadByUser;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(unreadByUser);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageRead(msgId, unreadByUser);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageReplied",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool replied;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(replied);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageReplied(msgId, replied);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageSend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		Rs::Msgs::MessageInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(info);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageSend(info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageStar",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool mark;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(mark);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageStar(msgId, mark);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageToDraft",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		Rs::Msgs::MessageInfo info;
		std::string msgParentId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(msgParentId);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageToDraft(info, msgParentId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/MessageToTrash",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		bool bTrash;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(bTrash);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->MessageToTrash(msgId, bTrash);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/removeMessageTagType",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		uint32_t tagId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(tagId);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->removeMessageTagType(tagId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/resetMessageStandardTagTypes",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		Rs::Msgs::MsgTagType tags;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsMsgs->resetMessageStandardTagTypes(tags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(tags);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/sendChat",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatId id;
		std::string msg;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(msg);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->sendChat(id, msg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/sendLobbyStatusPeerLeaving",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
		}


		// call retroshare C++ API
		rsMsgs->sendLobbyStatusPeerLeaving(lobby_id);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/sendMail",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		RsGxsId from;
		std::string subject;
		std::string mailBody;
		std::set<RsGxsId> to = std::set< RsGxsId >();
		std::set<RsGxsId> cc = std::set< RsGxsId >();
		std::set<RsGxsId> bcc = std::set< RsGxsId >();
		std::vector<FileInfo> attachments = std::vector< FileInfo >();
		std::set<RsMailIdRecipientIdPair> trackingIds = RS_DEFAULT_STORAGE_PARAM(std::set< RsMailIdRecipientIdPair >);
		std::string errorMsg = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(from);
			RS_SERIAL_PROCESS(subject);
			RS_SERIAL_PROCESS(mailBody);
			RS_SERIAL_PROCESS(to);
			RS_SERIAL_PROCESS(cc);
			RS_SERIAL_PROCESS(bcc);
			RS_SERIAL_PROCESS(attachments);
		}


		// call retroshare C++ API
		uint32_t retval = rsMsgs->sendMail(from, subject, mailBody, to, cc, bcc, attachments, trackingIds, errorMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(trackingIds);
			RS_SERIAL_PROCESS(errorMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/sendStatusString",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatId id;
		std::string status_string;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(status_string);
		}


		// call retroshare C++ API
		rsMsgs->sendStatusString(id, status_string);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setCustomStateString",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string status_string;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(status_string);
		}


		// call retroshare C++ API
		rsMsgs->setCustomStateString(status_string);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setDefaultIdentityForChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		RsGxsId nick;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(nick);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->setDefaultIdentityForChatLobby(nick);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setDistantMessagingPermissionFlags",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		uint32_t flags;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		rsMsgs->setDistantMessagingPermissionFlags(flags);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setIdentityForChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;
		RsGxsId nick;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
			RS_SERIAL_PROCESS(nick);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->setIdentityForChatLobby(lobby_id, nick);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setLobbyAutoSubscribe",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;
		bool autoSubscribe;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
			RS_SERIAL_PROCESS(autoSubscribe);
		}


		// call retroshare C++ API
		rsMsgs->setLobbyAutoSubscribe(lobby_id, autoSubscribe);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setMessageTag",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string msgId;
		uint32_t tagId;
		bool set;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(tagId);
			RS_SERIAL_PROCESS(set);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->setMessageTag(msgId, tagId, set);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/setMessageTagType",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		uint32_t tagId;
		std::string text;
		uint32_t rgb_color;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(tagId);
			RS_SERIAL_PROCESS(text);
			RS_SERIAL_PROCESS(rgb_color);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->setMessageTagType(tagId, text, rgb_color);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/SystemMessage",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		std::string title;
		std::string message;
		uint32_t systemFlag;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(title);
			RS_SERIAL_PROCESS(message);
			RS_SERIAL_PROCESS(systemFlag);
		}


		// call retroshare C++ API
		bool retval = rsMsgs->SystemMessage(title, message, systemFlag);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsMsgs/unsubscribeChatLobby",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsMsgs, "rsMsgs", cAns, session ) )
			return;

		ChatLobbyId lobby_id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(lobby_id);
		}


		// call retroshare C++ API
		rsMsgs->unsubscribeChatLobby(lobby_id);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/clearDistantSearchResults",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		TurtleRequestId reqId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(reqId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->clearDistantSearchResults(reqId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/createChannel",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsChannelGroup channel;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channel);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->createChannel(channel);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(channel);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/createChannelV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::string name;
		std::string description;
		RsGxsImage thumbnail = RsGxsImage();
		RsGxsId authorId = RsGxsId();
		RsGxsCircleType circleType = RsGxsCircleType::PUBLIC;
		RsGxsCircleId circleId = RsGxsCircleId();
		RsGxsGroupId channelId = RS_DEFAULT_STORAGE_PARAM(RsGxsGroupId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(name);
			RS_SERIAL_PROCESS(description);
			RS_SERIAL_PROCESS(thumbnail);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(circleType);
			RS_SERIAL_PROCESS(circleId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->createChannelV2(name, description, thumbnail, authorId, circleType, circleId, channelId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/createComment",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsComment comment;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(comment);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->createComment(comment);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(comment);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/createCommentV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		RsGxsMessageId threadId;
		std::string comment;
		RsGxsId authorId;
		RsGxsMessageId parentId = RsGxsMessageId();
		RsGxsMessageId origCommentId = RsGxsMessageId();
		RsGxsMessageId commentMessageId = RS_DEFAULT_STORAGE_PARAM(RsGxsMessageId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(threadId);
			RS_SERIAL_PROCESS(comment);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(parentId);
			RS_SERIAL_PROCESS(origCommentId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->createCommentV2(channelId, threadId, comment, authorId, parentId, origCommentId, commentMessageId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(commentMessageId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/createPostV2",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::string title;
		std::string mBody;
		std::list<RsGxsFile> files = std::list< RsGxsFile >();
		RsGxsImage thumbnail = RsGxsImage();
		RsGxsMessageId origPostId = RsGxsMessageId();
		RsGxsMessageId postId = RS_DEFAULT_STORAGE_PARAM(RsGxsMessageId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(title);
			RS_SERIAL_PROCESS(mBody);
			RS_SERIAL_PROCESS(files);
			RS_SERIAL_PROCESS(thumbnail);
			RS_SERIAL_PROCESS(origPostId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->createPostV2(channelId, title, mBody, files, thumbnail, origPostId, postId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(postId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/editChannel",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsChannelGroup channel;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channel);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->editChannel(channel);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/exportChannelLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::string link;
		RsGxsGroupId chanId;
		bool includeGxsData = true;
		std::string baseUrl = RsGxsChannels::DEFAULT_CHANNEL_BASE_URL;
		std::string errMsg = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(chanId);
			RS_SERIAL_PROCESS(includeGxsData);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->exportChannelLink(link, chanId, includeGxsData, baseUrl, errMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(errMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/ExtraFileHash",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::string path;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(path);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->ExtraFileHash(path);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/ExtraFileRemove",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsFileHash hash;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->ExtraFileRemove(hash);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelAllContent",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::vector<RsGxsChannelPost> posts;
		std::vector<RsGxsComment> comments;
		std::vector<RsGxsVote> votes;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelAllContent(channelId, posts, comments, votes);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(posts);
			RS_SERIAL_PROCESS(comments);
			RS_SERIAL_PROCESS(votes);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelAutoDownload",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		bool enabled;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelAutoDownload(channelId, enabled);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(enabled);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelComments",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::set<RsGxsMessageId> contentIds;
		std::vector<RsGxsComment> comments;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(contentIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelComments(channelId, contentIds, comments);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(comments);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelContent",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::set<RsGxsMessageId> contentsIds;
		std::vector<RsGxsChannelPost> posts;
		std::vector<RsGxsComment> comments;
		std::vector<RsGxsVote> votes;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(contentsIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelContent(channelId, contentsIds, posts, comments, votes);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(posts);
			RS_SERIAL_PROCESS(comments);
			RS_SERIAL_PROCESS(votes);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelDownloadDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::string directory;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelDownloadDirectory(channelId, directory);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(directory);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelGroupStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		GxsGroupStatistic stat;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelGroupStatistics(channelId, stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelServiceStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		GxsServiceStatistic stat;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelServiceStatistics(stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelsInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::list<RsGxsGroupId> chanIds;
		std::vector<RsGxsChannelGroup> channelsInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(chanIds);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelsInfo(chanIds, channelsInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(channelsInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelsSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::list<RsGroupMetaData> channels;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelsSummaries(channels);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(channels);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getChannelStatistics",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		RsGxsChannelStatistics stat;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getChannelStatistics(channelId, stat);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(stat);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getContentSummaries",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::vector<RsMsgMetaData> summaries;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getContentSummaries(channelId, summaries);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(summaries);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getDefaultStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsGxsChannels->getDefaultStoragePeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getDefaultSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsGxsChannels->getDefaultSyncPeriod();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getDistantSearchResultGroupData",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		RsGxsChannelGroup distantGroup;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->getDistantSearchResultGroupData(groupId, distantGroup);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(distantGroup);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsGxsChannels->getStoragePeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/getSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		uint32_t retval = rsGxsChannels->getSyncPeriod(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/importChannelLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::string link;
		RsGxsGroupId chanId = RS_DEFAULT_STORAGE_PARAM(RsGxsGroupId);
		std::string errMsg = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(link);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->importChannelLink(link, chanId, errMsg);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(chanId);
			RS_SERIAL_PROCESS(errMsg);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/requestStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		uint32_t token;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(token);
		}


		// call retroshare C++ API
		RsTokenService::GxsRequestStatus retval = rsGxsChannels->requestStatus(token);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/retrieveDistantSearchResults",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		TurtleRequestId searchId;
		std::map<RsGxsGroupId,RsGxsGroupSearchResults> results;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(searchId);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->retrieveDistantSearchResults(searchId, results);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(results);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/setChannelAutoDownload",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		bool enable;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(enable);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->setChannelAutoDownload(channelId, enable);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/setChannelDownloadDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::string directory;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(directory);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->setChannelDownloadDirectory(channelId, directory);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/setCommentReadStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGrpMsgIdPair msgId;
		bool read;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(read);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->setCommentReadStatus(msgId, read);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/setMessageReadStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGrpMsgIdPair msgId;
		bool read;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(msgId);
			RS_SERIAL_PROCESS(read);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->setMessageReadStatus(msgId, read);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/setStoragePeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t storageSecs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(storageSecs);
		}


		// call retroshare C++ API
		rsGxsChannels->setStoragePeriod(groupId, storageSecs);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/setSyncPeriod",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId groupId;
		uint32_t syncAge;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(syncAge);
		}


		// call retroshare C++ API
		rsGxsChannels->setSyncPeriod(groupId, syncAge);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/shareChannelKeys",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		std::set<RsPeerId> peers;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(peers);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->shareChannelKeys(channelId, peers);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/subscribeToChannel",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		bool subscribe;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(subscribe);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->subscribeToChannel(channelId, subscribe);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/turtleGroupRequest",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		TurtleRequestId retval = rsGxsChannels->turtleGroupRequest(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/turtleSearchRequest",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		std::string matchString;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(matchString);
		}


		// call retroshare C++ API
		TurtleRequestId retval = rsGxsChannels->turtleSearchRequest(matchString);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsGxsChannels/voteForComment",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsGxsChannels, "rsGxsChannels", cAns, session ) )
			return;

		RsGxsGroupId channelId;
		RsGxsMessageId postId;
		RsGxsMessageId commentId;
		RsGxsId authorId;
		RsGxsVoteType vote;
		RsGxsMessageId voteId = RS_DEFAULT_STORAGE_PARAM(RsGxsMessageId);
		std::string errorMessage = RS_DEFAULT_STORAGE_PARAM(std::string);


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(channelId);
			RS_SERIAL_PROCESS(postId);
			RS_SERIAL_PROCESS(commentId);
			RS_SERIAL_PROCESS(authorId);
			RS_SERIAL_PROCESS(vote);
		}


		// call retroshare C++ API
		bool retval = rsGxsChannels->voteForComment(channelId, postId, commentId, authorId, vote, voteId, errorMessage);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(voteId);
			RS_SERIAL_PROCESS(errorMessage);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/getAllBandwidthRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		std::map<RsPeerId,RsConfigDataRates> ratemap;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->getAllBandwidthRates(ratemap);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(ratemap);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/getConfigNetStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		RsConfigNetStatus status;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->getConfigNetStatus(status);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(status);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/GetCurrentDataRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		float inKb;
		float outKb;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->GetCurrentDataRates(inKb, outKb);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(inKb);
			RS_SERIAL_PROCESS(outKb);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/GetMaxDataRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		int inKb;
		int outKb;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->GetMaxDataRates(inKb, outKb);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(inKb);
			RS_SERIAL_PROCESS(outKb);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/getMaxDataRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		int inKb;
		int outKb;
		int inKbWhenIdle;
		int outKbWhenIdle;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->getMaxDataRates(inKb, outKb, inKbWhenIdle, outKbWhenIdle);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(inKb);
			RS_SERIAL_PROCESS(outKb);
			RS_SERIAL_PROCESS(inKbWhenIdle);
			RS_SERIAL_PROCESS(outKbWhenIdle);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/getOperatingMode",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		RsOpMode retval = rsConfig->getOperatingMode();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/getTotalBandwidthRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		RsConfigDataRates rates;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->getTotalBandwidthRates(rates);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(rates);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/getTrafficInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		std::list<RSTrafficClue> out_lst;
		std::list<RSTrafficClue> in_lst;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		int retval = rsConfig->getTrafficInfo(out_lst, in_lst);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(out_lst);
			RS_SERIAL_PROCESS(in_lst);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/setIsIdle",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		bool isIdle;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(isIdle);
		}


		// call retroshare C++ API
		rsConfig->setIsIdle(isIdle);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/SetMaxDataRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		int downKb;
		int upKb;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(downKb);
			RS_SERIAL_PROCESS(upKb);
		}


		// call retroshare C++ API
		int retval = rsConfig->SetMaxDataRates(downKb, upKb);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/setMaxDataRates",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		int inKb;
		int outKb;
		int inKbWhenIdle;
		int outKbWhenIdle;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(inKb);
			RS_SERIAL_PROCESS(outKb);
			RS_SERIAL_PROCESS(inKbWhenIdle);
			RS_SERIAL_PROCESS(outKbWhenIdle);
		}


		// call retroshare C++ API
		int retval = rsConfig->setMaxDataRates(inKb, outKb, inKbWhenIdle, outKbWhenIdle);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsConfig/setOperatingMode",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsConfig, "rsConfig", cAns, session ) )
			return;

		RsOpMode opMode;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(opMode);
		}


		// call retroshare C++ API
		bool retval = rsConfig->setOperatingMode(opMode);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/acceptInvite",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::string invite;
		ServicePermissionFlags flags = RS_NODE_PERM_DEFAULT;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(invite);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		bool retval = rsPeers->acceptInvite(invite, flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/addFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		RsPgpId gpgId;
		ServicePermissionFlags flags = RS_NODE_PERM_DEFAULT;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(gpgId);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		bool retval = rsPeers->addFriend(sslId, gpgId, flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/addGroup",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsGroupInfo groupInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupInfo);
		}


		// call retroshare C++ API
		bool retval = rsPeers->addGroup(groupInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/addPeerLocator",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		RsUrl locator;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(locator);
		}


		// call retroshare C++ API
		bool retval = rsPeers->addPeerLocator(sslId, locator);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/addSslOnlyFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		RsPgpId pgpId;
		RsPeerDetails details = RsPeerDetails();


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(pgpId);
			RS_SERIAL_PROCESS(details);
		}


		// call retroshare C++ API
		bool retval = rsPeers->addSslOnlyFriend(sslId, pgpId, details);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/assignPeersToGroup",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsNodeGroupId groupId;
		std::list<RsPgpId> peerIds;
		bool assign;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(peerIds);
			RS_SERIAL_PROCESS(assign);
		}


		// call retroshare C++ API
		bool retval = rsPeers->assignPeersToGroup(groupId, peerIds, assign);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/assignPeerToGroup",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsNodeGroupId groupId;
		RsPgpId peerId;
		bool assign;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(peerId);
			RS_SERIAL_PROCESS(assign);
		}


		// call retroshare C++ API
		bool retval = rsPeers->assignPeerToGroup(groupId, peerId, assign);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/connectAttempt",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->connectAttempt(sslId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/editGroup",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsNodeGroupId groupId;
		RsGroupInfo groupInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
			RS_SERIAL_PROCESS(groupInfo);
		}


		// call retroshare C++ API
		bool retval = rsPeers->editGroup(groupId, groupInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getFriendList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::list<RsPeerId> sslIds;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsPeers->getFriendList(sslIds);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(sslIds);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getGPGId",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		RsPgpId retval = rsPeers->getGPGId(sslId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getGroupInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsNodeGroupId groupId;
		RsGroupInfo groupInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->getGroupInfo(groupId, groupInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(groupInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getGroupInfoByName",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::string groupName;
		RsGroupInfo groupInfo;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupName);
		}


		// call retroshare C++ API
		bool retval = rsPeers->getGroupInfoByName(groupName, groupInfo);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(groupInfo);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getGroupInfoList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::list<RsGroupInfo> groupInfoList;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsPeers->getGroupInfoList(groupInfoList);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(groupInfoList);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getOnlineList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::list<RsPeerId> sslIds;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsPeers->getOnlineList(sslIds);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(sslIds);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getPeerDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		RsPeerDetails det;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->getPeerDetails(sslId, det);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(det);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getPeersCount",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		uint32_t peersCount;
		uint32_t onlinePeersCount;
		bool countLocations = true;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(countLocations);
		}


		// call retroshare C++ API
		bool retval = rsPeers->getPeersCount(peersCount, onlinePeersCount, countLocations);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(peersCount);
			RS_SERIAL_PROCESS(onlinePeersCount);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getPgpFriendList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::vector<RsPgpId> pgpIds;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsPeers->getPgpFriendList(pgpIds);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(pgpIds);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getProxyServer",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		uint32_t type;
		std::string addr;
		uint16_t port;
		uint32_t status_flags;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(type);
		}


		// call retroshare C++ API
		bool retval = rsPeers->getProxyServer(type, addr, port, status_flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(addr);
			RS_SERIAL_PROCESS(port);
			RS_SERIAL_PROCESS(status_flags);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/GetRetroshareInvite",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId = RsPeerId();
		RetroshareInviteFlags inviteFlags = RsPeers::defaultCertificateFlags;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(inviteFlags);
		}


		// call retroshare C++ API
		std::string retval = rsPeers->GetRetroshareInvite(sslId, inviteFlags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/getShortInvite",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::string invite;
		RsPeerId sslId = RsPeerId();
		RetroshareInviteFlags inviteFlags = RsPeers::defaultCertificateFlags;
		std::string baseUrl = "https://me.retroshare.cc/";


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(inviteFlags);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		bool retval = rsPeers->getShortInvite(invite, sslId, inviteFlags, baseUrl);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(invite);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/isFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->isFriend(sslId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/isOnline",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->isOnline(sslId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/isPgpFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPgpId pgpId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(pgpId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->isPgpFriend(pgpId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/isSslOnlyFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->isSslOnlyFriend(sslId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/loadCertificateFromString",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::string cert;
		RsPeerId sslId;
		RsPgpId pgpId;
		std::string errorString;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(cert);
		}


		// call retroshare C++ API
		bool retval = rsPeers->loadCertificateFromString(cert, sslId, pgpId, errorString);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(pgpId);
			RS_SERIAL_PROCESS(errorString);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/loadDetailsFromStringCert",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::string cert;
		RsPeerDetails certDetails;
		uint32_t errorCode;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(cert);
		}


		// call retroshare C++ API
		bool retval = rsPeers->loadDetailsFromStringCert(cert, certDetails, errorCode);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(certDetails);
			RS_SERIAL_PROCESS(errorCode);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/parseShortInvite",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		std::string invite;
		RsPeerDetails details;
		uint32_t err_code;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(invite);
		}


		// call retroshare C++ API
		bool retval = rsPeers->parseShortInvite(invite, details, err_code);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(details);
			RS_SERIAL_PROCESS(err_code);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/pgpIdFromFingerprint",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPgpFingerprint fpr;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(fpr);
		}


		// call retroshare C++ API
		RsPgpId retval = rsPeers->pgpIdFromFingerprint(fpr);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/removeFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPgpId pgpId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(pgpId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->removeFriend(pgpId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/removeFriendLocation",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->removeFriendLocation(sslId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/removeGroup",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsNodeGroupId groupId;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(groupId);
		}


		// call retroshare C++ API
		bool retval = rsPeers->removeGroup(groupId);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/setDynDNS",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		std::string addr;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(addr);
		}


		// call retroshare C++ API
		bool retval = rsPeers->setDynDNS(sslId, addr);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/setExtAddress",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		std::string addr;
		uint16_t port;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(addr);
			RS_SERIAL_PROCESS(port);
		}


		// call retroshare C++ API
		bool retval = rsPeers->setExtAddress(sslId, addr, port);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/setLocalAddress",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		std::string addr;
		uint16_t port;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(addr);
			RS_SERIAL_PROCESS(port);
		}


		// call retroshare C++ API
		bool retval = rsPeers->setLocalAddress(sslId, addr, port);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/setNetworkMode",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		uint32_t netMode;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(netMode);
		}


		// call retroshare C++ API
		bool retval = rsPeers->setNetworkMode(sslId, netMode);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/setProxyServer",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		uint32_t type;
		std::string addr;
		uint16_t port;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(type);
			RS_SERIAL_PROCESS(addr);
			RS_SERIAL_PROCESS(port);
		}


		// call retroshare C++ API
		bool retval = rsPeers->setProxyServer(type, addr, port);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsPeers/setVisState",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsPeers, "rsPeers", cAns, session ) )
			return;

		RsPeerId sslId;
		uint16_t vsDisc;
		uint16_t vsDht;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(sslId);
			RS_SERIAL_PROCESS(vsDisc);
			RS_SERIAL_PROCESS(vsDht);
		}


		// call retroshare C++ API
		bool retval = rsPeers->setVisState(sslId, vsDisc, vsDht);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/autoPositiveOpinionForContacts",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsReputations->autoPositiveOpinionForContacts();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/banNode",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsPgpId id;
		bool b;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(b);
		}


		// call retroshare C++ API
		rsReputations->banNode(id, b);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/getOwnOpinion",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsGxsId id;
		RsOpinion op;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsReputations->getOwnOpinion(id, op);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(op);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/getReputationInfo",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsGxsId id;
		RsPgpId ownerNode;
		RsReputationInfo info;
		bool stamp = true;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(ownerNode);
			RS_SERIAL_PROCESS(stamp);
		}


		// call retroshare C++ API
		bool retval = rsReputations->getReputationInfo(id, ownerNode, info, stamp);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/isIdentityBanned",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsReputations->isIdentityBanned(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/isNodeBanned",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsPgpId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		bool retval = rsReputations->isNodeBanned(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/overallReputationLevel",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsGxsId id;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
		}


		// call retroshare C++ API
		RsReputationLevel retval = rsReputations->overallReputationLevel(id);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/rememberBannedIdThreshold",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsReputations->rememberBannedIdThreshold();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/setAutoPositiveOpinionForContacts",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		bool b;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(b);
		}


		// call retroshare C++ API
		rsReputations->setAutoPositiveOpinionForContacts(b);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/setOwnOpinion",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		RsGxsId id;
		RsOpinion op;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(id);
			RS_SERIAL_PROCESS(op);
		}


		// call retroshare C++ API
		bool retval = rsReputations->setOwnOpinion(id, op);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/setRememberBannedIdThreshold",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		uint32_t days;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(days);
		}


		// call retroshare C++ API
		rsReputations->setRememberBannedIdThreshold(days);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/setThresholdForRemotelyNegativeReputation",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		uint32_t thresh;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(thresh);
		}


		// call retroshare C++ API
		rsReputations->setThresholdForRemotelyNegativeReputation(thresh);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/setThresholdForRemotelyPositiveReputation",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;

		uint32_t thresh;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(thresh);
		}


		// call retroshare C++ API
		rsReputations->setThresholdForRemotelyPositiveReputation(thresh);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/thresholdForRemotelyNegativeReputation",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsReputations->thresholdForRemotelyNegativeReputation();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsReputations/thresholdForRemotelyPositiveReputation",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsReputations, "rsReputations", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsReputations->thresholdForRemotelyPositiveReputation();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/addSharedDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		SharedDirInfo dir;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(dir);
		}


		// call retroshare C++ API
		bool retval = rsFiles->addSharedDirectory(dir);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/alreadyHaveFile",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		FileInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsFiles->alreadyHaveFile(hash, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/banFile",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash realFileHash;
		std::string filename;
		uint64_t fileSize;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(realFileHash);
			RS_SERIAL_PROCESS(filename);
			RS_SERIAL_PROCESS(fileSize);
		}


		// call retroshare C++ API
		int retval = rsFiles->banFile(realFileHash, filename, fileSize);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/defaultChunkStrategy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		FileChunksInfo::ChunkStrategy retval = rsFiles->defaultChunkStrategy();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/defaultEncryptionPolicy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsFiles->defaultEncryptionPolicy();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/exportCollectionLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string link;
		uint64_t handle;
		bool fragSneak = false;
		std::string baseUrl = RsFiles::DEFAULT_FILES_BASE_URL;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(handle);
			RS_SERIAL_PROCESS(fragSneak);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		std::error_condition retval = rsFiles->exportCollectionLink(link, handle, fragSneak, baseUrl);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/exportFileLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string link;
		RsFileHash fileHash;
		uint64_t fileSize;
		std::string fileName;
		bool fragSneak = false;
		std::string baseUrl = RsFiles::DEFAULT_FILES_BASE_URL;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(fileHash);
			RS_SERIAL_PROCESS(fileSize);
			RS_SERIAL_PROCESS(fileName);
			RS_SERIAL_PROCESS(fragSneak);
			RS_SERIAL_PROCESS(baseUrl);
		}


		// call retroshare C++ API
		std::error_condition retval = rsFiles->exportFileLink(link, fileHash, fileSize, fileName, fragSneak, baseUrl);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(link);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/ExtraFileHash",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string localpath;
		rstime_t period;
		TransferRequestFlags flags;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(localpath);
			RS_SERIAL_PROCESS(period);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		bool retval = rsFiles->ExtraFileHash(localpath, period, flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/ExtraFileRemove",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsFiles->ExtraFileRemove(hash);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/ExtraFileStatus",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string localpath;
		FileInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(localpath);
		}


		// call retroshare C++ API
		bool retval = rsFiles->ExtraFileStatus(localpath, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileCancel",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsFiles->FileCancel(hash);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileClearCompleted",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsFiles->FileClearCompleted();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileControl",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		uint32_t flags;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		bool retval = rsFiles->FileControl(hash, flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		FileSearchFlags hintflags;
		FileInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(hintflags);
		}


		// call retroshare C++ API
		bool retval = rsFiles->FileDetails(hash, hintflags, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileDownloadChunksDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		FileChunksInfo info;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsFiles->FileDownloadChunksDetails(hash, info);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(info);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileDownloads",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::list<RsFileHash> hashs;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		rsFiles->FileDownloads(hashs);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(hashs);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/filePermDirectDL",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsFiles->filePermDirectDL();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileRequest",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string fileName;
		RsFileHash hash;
		uint64_t size;
		std::string destPath;
		TransferRequestFlags flags;
		std::list<RsPeerId> srcIds;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(fileName);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(size);
			RS_SERIAL_PROCESS(destPath);
			RS_SERIAL_PROCESS(flags);
			RS_SERIAL_PROCESS(srcIds);
		}


		// call retroshare C++ API
		bool retval = rsFiles->FileRequest(fileName, hash, size, destPath, flags, srcIds);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileUploadChunksDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		RsPeerId peerId;
		CompressedChunkMap map;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(peerId);
		}


		// call retroshare C++ API
		bool retval = rsFiles->FileUploadChunksDetails(hash, peerId, map);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(map);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/FileUploads",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::list<RsFileHash> hashs;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsFiles->FileUploads(hashs);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(hashs);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/ForceDirectoryCheck",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		bool add_safe_delay = false;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(add_safe_delay);
		}


		// call retroshare C++ API
		rsFiles->ForceDirectoryCheck(add_safe_delay);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/freeDiskSpaceLimit",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsFiles->freeDiskSpaceLimit();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getChunkStrategy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		FileChunksInfo::ChunkStrategy s;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsFiles->getChunkStrategy(hash, s);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(s);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getDownloadDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::string retval = rsFiles->getDownloadDirectory();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getMaxUploadSlotsPerFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsFiles->getMaxUploadSlotsPerFriend();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getPartialsDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		std::string retval = rsFiles->getPartialsDirectory();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getPrimaryBannedFilesList",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::map<RsFileHash,BannedFileEntry> bannedFiles;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsFiles->getPrimaryBannedFilesList(bannedFiles);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(bannedFiles);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getQueueSize",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;



		// deserialize input parameters from JSON


		// call retroshare C++ API
		uint32_t retval = rsFiles->getQueueSize();


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/getSharedDirectories",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::list<SharedDirInfo> dirs;


		// deserialize input parameters from JSON


		// call retroshare C++ API
		bool retval = rsFiles->getSharedDirectories(dirs);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(dirs);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/isHashBanned",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
		}


		// call retroshare C++ API
		bool retval = rsFiles->isHashBanned(hash);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/parseFilesLink",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string link;
		RsFileTree collection;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(link);
		}


		// call retroshare C++ API
		std::error_condition retval = rsFiles->parseFilesLink(link, collection);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(collection);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/removeSharedDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string dir;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(dir);
		}


		// call retroshare C++ API
		bool retval = rsFiles->removeSharedDirectory(dir);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/requestDirDetails",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		DirDetails details;
		uint64_t handle = 0;
		FileSearchFlags flags = RS_FILE_HINTS_LOCAL;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(handle);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		bool retval = rsFiles->requestDirDetails(details, handle, flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(details);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/requestFiles",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileTree collection;
		std::string destPath = "";
		std::vector<RsPeerId> srcIds = std::vector< RsPeerId >();
		FileRequestFlags flags = FileRequestFlags::ANONYMOUS_ROUTING;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(collection);
			RS_SERIAL_PROCESS(destPath);
			RS_SERIAL_PROCESS(srcIds);
			RS_SERIAL_PROCESS(flags);
		}


		// call retroshare C++ API
		std::error_condition retval = rsFiles->requestFiles(collection, destPath, srcIds, flags);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setChunkStrategy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		FileChunksInfo::ChunkStrategy newStrategy;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(newStrategy);
		}


		// call retroshare C++ API
		bool retval = rsFiles->setChunkStrategy(hash, newStrategy);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setDefaultChunkStrategy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		FileChunksInfo::ChunkStrategy strategy;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(strategy);
		}


		// call retroshare C++ API
		rsFiles->setDefaultChunkStrategy(strategy);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setDefaultEncryptionPolicy",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		uint32_t policy;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(policy);
		}


		// call retroshare C++ API
		rsFiles->setDefaultEncryptionPolicy(policy);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setDestinationDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		std::string newPath;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(newPath);
		}


		// call retroshare C++ API
		bool retval = rsFiles->setDestinationDirectory(hash, newPath);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setDestinationName",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash hash;
		std::string newName;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(hash);
			RS_SERIAL_PROCESS(newName);
		}


		// call retroshare C++ API
		bool retval = rsFiles->setDestinationName(hash, newName);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setDownloadDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string path;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(path);
		}


		// call retroshare C++ API
		bool retval = rsFiles->setDownloadDirectory(path);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setFilePermDirectDL",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		uint32_t perm;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(perm);
		}


		// call retroshare C++ API
		rsFiles->setFilePermDirectDL(perm);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setFreeDiskSpaceLimit",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		uint32_t minimumFreeMB;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(minimumFreeMB);
		}


		// call retroshare C++ API
		rsFiles->setFreeDiskSpaceLimit(minimumFreeMB);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setMaxUploadSlotsPerFriend",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		uint32_t n;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(n);
		}


		// call retroshare C++ API
		rsFiles->setMaxUploadSlotsPerFriend(n);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setPartialsDirectory",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string path;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(path);
		}


		// call retroshare C++ API
		bool retval = rsFiles->setPartialsDirectory(path);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setQueueSize",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		uint32_t s;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(s);
		}


		// call retroshare C++ API
		rsFiles->setQueueSize(s);


		// serialize out parameters and return value to JSON


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/setSharedDirectories",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::list<SharedDirInfo> dirs;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(dirs);
		}


		// call retroshare C++ API
		bool retval = rsFiles->setSharedDirectories(dirs);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/turtleSearch",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string matchString;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(matchString);
		}


		// call retroshare C++ API
		TurtleRequestId retval = rsFiles->turtleSearch(matchString);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License version 3 as    *
 * published by the Free Software Foundation.                                  *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/turtleSearchRequest",
                 [this](const std::shared_ptr<rb::Session> session)
{
	const std::multimap<std::string, std::string> headers
	{
		{ "Connection", "keep-alive" },
		{ "Content-Type", "text/event-stream" }
	};
	session->yield(rb::OK, headers);

	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [this](
					const std::shared_ptr<rb::Session> session,
					const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		std::string matchString;
		std::function< void(const std::vector< TurtleFileInfoV2 > &results)>  multiCallback;
		rstime_t maxWait = 300;


		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(matchString);
			RS_SERIAL_PROCESS(maxWait);
		}


		const std::weak_ptr<rb::Service> weakService(mService);
		const std::weak_ptr<rb::Session> weakSession(session);
		multiCallback = [weakService, weakSession](const std::vector< TurtleFileInfoV2 > &results)
		{
			auto session = weakSession.lock();
			if(!session || session->is_closed()) return;

			auto lService = weakService.lock();
			if(!lService || lService->is_down()) return;

			RsGenericSerializer::SerializeContext ctx;
			RsTypeSerializer::serial_process(RsGenericSerializer::TO_JSON, ctx, const_cast<std::vector< TurtleFileInfoV2 > &>(results), "results" );


			std::stringstream sStream;
			sStream << "data: " << compactJSON << ctx.mJson << "\n\n";
			const std::string message = sStream.str();

			lService->schedule( [weakSession, message]()
			{
				auto session = weakSession.lock();
				if(!session || session->is_closed()) return;
				session->yield(message);
				
			} );
		};

		bool retval = rsFiles->turtleSearchRequest(matchString, multiCallback, maxWait);


		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		std::stringstream message;
		message << "data: " << compactJSON << cAns.mJson << "\n\n";
		session->yield(message.str());
		RsThread::async( [=](){std::this_thread::sleep_for(std::chrono::seconds(maxWait+120) );auto lService = weakService.lock();if(!lService || lService->is_down()) return;lService->schedule( [=](){auto session = weakSession.lock();if(session && session->is_open())session->close();} );} );
	} );
}, true );
/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/unbanFile",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		RsFileHash realFileHash;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(realFileHash);
		}


		// call retroshare C++ API
		int retval = rsFiles->unbanFile(realFileHash);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

/*******************************************************************************
 * RetroShare JSON API                                                         *
 *                                                                             *
 * Copyright (C) 2018-2019  Gioacchino Mazzurco <gio@eigenlab.org>             *
 *                                                                             *
 * This program is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU Affero General Public License as              *
 * published by the Free Software Foundation, either version 3 of the          *
 * License, or (at your option) any later version.                             *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU Affero General Public License for more details.                         *
 *                                                                             *
 * You should have received a copy of the GNU Affero General Public License    *
 * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
 *                                                                             *
 *******************************************************************************/

registerHandler( "/rsFiles/updateShareFlags",
                 [](const std::shared_ptr<rb::Session> session)
{
	size_t reqSize = session->get_request()->get_header("Content-Length", 0);
	session->fetch( reqSize, [](
	                const std::shared_ptr<rb::Session> session,
	                const rb::Bytes& body )
	{
		INITIALIZE_API_CALL_JSON_CONTEXT;

		if( !checkRsServicePtrReady(
		            rsFiles, "rsFiles", cAns, session ) )
			return;

		SharedDirInfo dir;


		// deserialize input parameters from JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cReq);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::FROM_JSON);
			RS_SERIAL_PROCESS(dir);
		}


		// call retroshare C++ API
		bool retval = rsFiles->updateShareFlags(dir);


		// serialize out parameters and return value to JSON
		{
			RsGenericSerializer::SerializeContext& ctx(cAns);
			RsGenericSerializer::SerializeJob j(RsGenericSerializer::TO_JSON);
			RS_SERIAL_PROCESS(retval);
		}


		// return them to the API caller
		DEFAULT_API_CALL_JSON_RETURN(rb::OK);
	} );
}, true );

